@@ -21,7 +21,10 @@ func TestParse(t *testing.T) {
 		// grammar
 		{"", ""},
 		{"\n", ""},
+		{"#!/usr/bin/env node", "Stmt(#!/usr/bin/env node)"},
 		{"/* comment */", ""},
+		{"/*! comment */", "Stmt(/*! comment */)"},
+		{"5 + /*! comment */ 6", "Stmt(5+6)"},
 		{"{}", "Stmt({ })"},
 		{`"use strict"`, `Stmt("use strict")`},
 		{"var a = b;", "Decl(var Binding(a = b))"},
@@ -38,20 +41,18 @@ func TestParse(t *testing.T) {
 		{"var a = 5 * 4 / 3 ** 2 + ( 5 - 3 );", "Decl(var Binding(a = (((5*4)/(3**2))+((5-3)))))"},
 		{"var a, b = c;", "Decl(var Binding(a) Binding(b = c))"},
 		{"var a,\nb = c;", "Decl(var Binding(a) Binding(b = c))"},
-		{";", "Stmt(;)"},
-		{"{; var a = 3;}", "Stmt({ Stmt(;) Decl(var Binding(a = 3)) })"},
+		{";", "Stmt()"},
+		{"{; var a = 3;}", "Stmt({ Stmt() Decl(var Binding(a = 3)) })"},
 		{"{a=5}", "Stmt({ Stmt(a=5) })"},
-		{"return", "Stmt(return)"},
-		{"return 5*3", "Stmt(return (5*3))"},
 		{"break", "Stmt(break)"},
 		{"break LABEL", "Stmt(break LABEL)"},
 		{"continue", "Stmt(continue)"},
 		{"continue LABEL", "Stmt(continue LABEL)"},
-		{"if (a == 5) return true", "Stmt(if (a==5) Stmt(return true))"},
-		{"if (a == 5) return true else return false", "Stmt(if (a==5) Stmt(return true) else Stmt(return false))"},
+		{"if (a == 5) continue LABEL", "Stmt(if (a==5) Stmt(continue LABEL))"},
+		{"if (a == 5) continue LABEL else continue LABEL", "Stmt(if (a==5) Stmt(continue LABEL) else Stmt(continue LABEL))"},
 		{"if (a) b; else if (c) d;", "Stmt(if a Stmt(b) else Stmt(if c Stmt(d)))"},
 		{"if (a) 1; else if (b) 2; else 3", "Stmt(if a Stmt(1) else Stmt(if b Stmt(2) else Stmt(3)))"},
-		{"with (a = 5) return true", "Stmt(with (a=5) Stmt(return true))"},
+		{"with (a = 5) continue LABEL", "Stmt(with (a=5) Stmt(continue LABEL))"},
 		{"do a++; while (a < 4)", "Stmt(do Stmt(a++) while (a<4))"},
 		{"do {a++} while (a < 4)", "Stmt(do Stmt({ Stmt(a++) }) while (a<4))"},
 		{"while (a < 4) a++", "Stmt(while (a<4) Stmt(a++))"},
@@ -88,6 +89,7 @@ func TestParse(t *testing.T) {
 		{"function a(...b) {}", "Decl(function a Params(...Binding(b)) Stmt({ }))"},
 		{"function a(b, ...c) {}", "Decl(function a Params(Binding(b), ...Binding(c)) Stmt({ }))"},
 		{"function a(b) {return}", "Decl(function a Params(Binding(b)) Stmt({ Stmt(return) }))"},
+		{"function a(){return 5*3}", "Decl(function a Params() Stmt({ Stmt(return (5*3)) }))"},
 		{"class A {}", "Decl(class A)"},
 		{"class A {;}", "Decl(class A)"},
 		{"!class{}", "Stmt(!Decl(class))"},
@@ -115,7 +117,7 @@ func TestParse(t *testing.T) {
 		{"class A { static #field=5 }", "Decl(class A Field(static #field = 5))"},
 		{"class A { #method(){} }", "Decl(class A Method(#method Params() Stmt({ })))"},
 		{"class A { get }", "Decl(class A Field(get))"},
-		{"class A { field static get method(){} }", "Decl(class A Field(field) Method(static get method Params() Stmt({ })))"},
+		{"class A { field static get method(){ return 5 } }", "Decl(class A Field(field) Method(static get method Params() Stmt({ Stmt(return 5) })))"},
 		{"class A { static { this.field = 5 } }", "Decl(class A Static(Stmt({ Stmt((this.field)=5) })))"},
 		//{"class A { get get get(){} }", "Decl(class A Definition(get) Method(get get Params() Stmt({ })))"}, // doesn't look like this should be supported
 		{"`tmpl`", "Stmt(`tmpl`)"},
@@ -156,6 +158,7 @@ func TestParse(t *testing.T) {
 		{"async a => b", "Stmt(async Params(Binding(a)) => Stmt({ Stmt(return b) }))"},
 		{"async (a) => b", "Stmt(async Params(Binding(a)) => Stmt({ Stmt(return b) }))"},
 		{"async(a)", "Stmt(async(a))"},
+		{"async();", "Stmt(async())"},
 		{"async(a=6, ...b)", "Stmt(async((a=6), ...b))"},
 		{"async(function(){})", "Stmt(async(Decl(function Params() Stmt({ }))))"},
 		{"function a(){ async\nawait => b }", "Decl(function a Params() Stmt({ Stmt(async) Stmt(Params(Binding(await)) => Stmt({ Stmt(return b) })) }))"},
@@ -173,6 +176,7 @@ func TestParse(t *testing.T) {
 		{"function*a(){ (yield) }", "Decl(function* a Params() Stmt({ Stmt((yield)) }))"},
 		{"function*a(){ (yield a) }", "Decl(function* a Params() Stmt({ Stmt((yield a)) }))"},
 		{"function a(){ let\nawait }", "Decl(function a Params() Stmt({ Decl(let Binding(await)) }))"},
+		{"function*a(){ b => yield%5 }", "Decl(function* a Params() Stmt({ Stmt(Params(Binding(b)) => Stmt({ Stmt(return (yield%5)) })) }))"},
 		{"x = {await}", "Stmt(x={await})"},
 		{"async function a(){ x = {await: 5} }", "Decl(async function a Params() Stmt({ Stmt(x={await: 5}) }))"},
 		{"async function a(){ x = await a }", "Decl(async function a Params() Stmt({ Stmt(x=(await a)) }))"},
@@ -246,13 +250,14 @@ func TestParse(t *testing.T) {
 		{"x = {async*a(){}}", "Stmt(x={Method(async * a Params() Stmt({ }))})"},
 		{"x = {get a(){}}", "Stmt(x={Method(get a Params() Stmt({ }))})"},
 		{"x = {set a(){}}", "Stmt(x={Method(set a Params() Stmt({ }))})"},
-		{"x = {get(){}}", "Stmt(x={Method(get Params() Stmt({ }))})"},
+		{"x = {get(){ return 5 }}", "Stmt(x={Method(get Params() Stmt({ Stmt(return 5) }))})"},
 		{"x = {set(){}}", "Stmt(x={Method(set Params() Stmt({ }))})"},
 		{"x = (a, b)", "Stmt(x=((a,b)))"},
 		{"x = function() {}", "Stmt(x=Decl(function Params() Stmt({ })))"},
 		{"x = async function() {}", "Stmt(x=Decl(async function Params() Stmt({ })))"},
 		{"x = class {}", "Stmt(x=Decl(class))"},
 		{"x = class {a(){}}", "Stmt(x=Decl(class Method(a Params() Stmt({ }))))"},
+		{"x = class A { [(0())]; };", "Stmt(x=Decl(class A Field([(0())])))"},
 		{"x = a => a++", "Stmt(x=(Params(Binding(a)) => Stmt({ Stmt(return (a++)) })))"},
 		{"x = a => {a++}", "Stmt(x=(Params(Binding(a)) => Stmt({ Stmt(a++) })))"},
 		{"x = a => {return}", "Stmt(x=(Params(Binding(a)) => Stmt({ Stmt(return) })))"},
@@ -312,6 +317,7 @@ func TestParse(t *testing.T) {
 		{"protected = 0", "Stmt(protected=0)"},
 		{"public = 0", "Stmt(public=0)"},
 		{"static = 0", "Stmt(static=0)"},
+		{"#x in a", "Stmt(#x in a)"},
 
 		// expression to arrow function parameters
 		{"x = (a,b,c) => {a++}", "Stmt(x=(Params(Binding(a), Binding(b), Binding(c)) => Stmt({ Stmt(a++) })))"},
@@ -327,6 +333,7 @@ func TestParse(t *testing.T) {
 		{"x = ([{a: b}]) => {a++}", "Stmt(x=(Params(Binding([ Binding({ a: Binding(b) }) ])) => Stmt({ Stmt(a++) })))"},
 		{"x = (a = 5) => {a++}", "Stmt(x=(Params(Binding(a = 5)) => Stmt({ Stmt(a++) })))"},
 		{"x = ({a = 5}) => {a++}", "Stmt(x=(Params(Binding({ Binding(a = 5) })) => Stmt({ Stmt(a++) })))"},
+		{`({a:b=1}={})=>b`, `Stmt(Params(Binding({ a: Binding(b = 1) } = {})) => Stmt({ Stmt(return b) }))`},
 
 		// expression precedence
 		{"!!a", "Stmt(!(!a))"},
@@ -356,7 +363,7 @@ func TestParse(t *testing.T) {
 
 		// regular expressions
 		{"/abc/", "Stmt(/abc/)"},
-		{"return /abc/;", "Stmt(return /abc/)"},
+		{"function f(){return /abc/;}", "Decl(function f Params() Stmt({ Stmt(return /abc/) }))"},
 		{"a/b/g", "Stmt((a/b)/g)"},
 		{"{}/1/g", "Stmt({ }) Stmt(/1/g)"},
 		{"i(0)/1/g", "Stmt(((i(0))/1)/g)"},
@@ -389,12 +396,12 @@ func TestParse(t *testing.T) {
 		{"try{}catch(a){var a}", "Stmt(try Stmt({ }) catch Binding(a) Stmt({ Decl(var Binding(a)) }))"},
 
 		// ASI
-		{"return a", "Stmt(return a)"},
-		{"return; a", "Stmt(return) Stmt(a)"},
-		{"return\na", "Stmt(return) Stmt(a)"},
-		{"return /*comment*/ a", "Stmt(return a)"},
-		{"return /*com\nment*/ a", "Stmt(return) Stmt(a)"},
-		{"return //comment\n a", "Stmt(return) Stmt(a)"},
+		{"function f(){return a}", "Decl(function f Params() Stmt({ Stmt(return a) }))"},
+		{"function f(){return; a}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
+		{"function f(){return\na}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
+		{"function f(){return /*comment*/ a}", "Decl(function f Params() Stmt({ Stmt(return a) }))"},
+		{"function f(){return /*com\nment*/ a}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
+		{"function f(){return //comment\n a}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
 		{"a?.b\n`c`", "Stmt((a?.b)`c`)"},
 
 		{"() => { const v=6; x={v} }", "Stmt(Params() => Stmt({ Decl(const Binding(v = 6)) Stmt(x={v}) }))"},
@@ -447,6 +454,7 @@ func TestParseError(t *testing.T) {
 		{"for(a;a;a", "expected ) instead of EOF in for statement"},
 		{"for(var [a],b;", "unexpected ; in for statement"},
 		{"for(var [a]=5,{b};", "expected = instead of ; in var statement"},
+		{`for(let{[(`, `unexpected EOF in expression`},
 		{"for await", "expected ( instead of EOF in for statement"},
 		{"function a(){for await", "expected ( instead of await in for statement"},
 		{"async function a(){ for await(a;", "expected of instead of ; in for statement"},
@@ -539,6 +547,7 @@ func TestParseError(t *testing.T) {
 		{"import {yield as", "expected Identifier instead of EOF in import statement"},
 		{"import {yield,", "expected } instead of EOF in import statement"},
 		{"import yield", "expected from instead of EOF in import statement"},
+		{"import yield,", "expected * or { instead of EOF in import statement"},
 		{"import yield from", "expected String instead of EOF in import statement"},
 		{"export", "expected *, {, var, let, const, function, async, class, or default instead of EOF in export statement"},
 		{"export *", "expected from instead of EOF in export statement"},
@@ -551,6 +560,8 @@ func TestParseError(t *testing.T) {
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export async", "expected function instead of EOF in export statement"},
+		{"throw", "unexpected EOF in expression"},
+		{"throw\n", "unexpected newline in throw statement"},
 
 		// no declarations
 		{"if(a) function f(){}", "unexpected function in statement"},
@@ -564,7 +575,7 @@ func TestParseError(t *testing.T) {
 		{"function a(b = yield c){}", "unexpected c in function declaration"},
 		{"function*a(){ (yield) => yield }", "unexpected => in expression"},
 		{"function*a(){ (yield=5) => yield }", "unexpected = in expression"},
-		{"function*a(){ (...yield) => yield }", "unexpected yield in arrow function"},
+		{"function*a(){ (...yield) => yield }", "unexpected yield in expression"},
 		{"x = await\n=> a++", "unexpected => in expression"},
 		{"x=async (await,", "unexpected , in expression"},
 		{"async function a() { class a extends await", "unexpected await in expression"},
@@ -575,7 +586,7 @@ func TestParseError(t *testing.T) {
 		{"async function a() { x = await =>", "unexpected => in expression"},
 		{"async function a(){ (await) => await }", "unexpected ) in expression"},
 		{"async function a(){ (await=5) => await }", "unexpected = in expression"},
-		{"async function a(){ (...await) => await }", "unexpected await in arrow function"},
+		{"async function a(){ (...await) => await }", "unexpected await in expression"},
 		{"async+a b", "unexpected b in expression"},
 		{"(async\nfunction(){})", "unexpected function in expression"},
 		{"a + async b", "unexpected b in expression"},
@@ -586,7 +597,7 @@ func TestParseError(t *testing.T) {
 		{"let\nawait", "unexpected await in binding"},
 		{"let {await = 5} = z", "expected : instead of = in object binding"},
 		{"x = await => a++", "unexpected => in expression"},
-		{"function*a(){ b => yield%5 }", "unexpected % in expression"},
+		{"function*a(){ b => yield 0 }", "unexpected 0 in expression"},
 
 		// specific cases
 		{"{a, if: b, do(){}, ...d}", "unexpected if in expression"}, // block stmt
@@ -627,6 +638,8 @@ func TestParseError(t *testing.T) {
 		{"x = (a, a) =>", "unexpected => in expression"},
 		{"x = (a, a, ...a) =>", "unexpected ... in expression"},
 		{"x = (a, ...a) =>", "identifier a has already been declared"},
+		{"(A,{}%0%{})=>0", "invalid parameters in arrow function"},
+		{"({}``=1)=>0", "invalid parameters in arrow function"},
 
 		// expression precedence
 		{"x = a + yield b", "unexpected b in expression"},
